      * Demonstration of fetching data into an array of data structures
      * using the SQL CLI API.
      *                                    Scott Klement, July 27, 2006
      *
      * To compile:
      *    first, create the CUSTFILE table and put some data into it.
      *    CRTBNDRPG CLIARRAY SRCFILE(xxx/QRPGLESRC) DBGVIEW(*LIST)
      *
      * To run:
      *    CALL CLIARRAY
      *
      * Note that the printout is intended to be for demonstration
      *      purposes only, and therefore does not print headings
      *      or do page breaks.  I suggest that you put it on hold
      *      and view it in the spool rather than printing it.
      *
     H DFTACTGRP(*NO) ACTGRP('SQLCLI')

     FQSYSPRT   O    F  132        PRINTER OFLIND(Overflow)

      /copy SQLCLI_H

     D CEE4RAGE        PR
     D   procedure                     *   procptr const
     D   feedback                    12A   options(*omit)

     D check_error     pr
     D    rc                         10I 0 value
     D    env                              like(SQLHENV)  value
     D    conn                             like(SQLHDBC)  value
     D    stmt                             like(SQLHSTMT) value
     D    apiname                    50A   varying const
     D cleanup         pr

     D FETCH_SIZE      C                   const(10)

     D env             s                   like(SQLHENV)
     D                                     inz(SQL_NULL_HENV)
     D conn            s                   like(SQLHDBC)
     D                                     inz(SQL_NULL_HDBC)
     D stmt            s                   like(SQLHSTMT)
     D                                     inz(SQL_NULL_HSTMT)

     D count           s                   like(SQLINTEGER)
     D status          s                   like(SQLSMALLINT)
     D                                     dim(FETCH_SIZE)
     D rc              s                   like(SQLRETURN)


     D PrintMe         ds           132    qualified
     D   City                        15A
     D                                1A
     D   Cust                         5A
     D                                1A
     D   Name                        25A

     D Row             ds                  qualified
     D                                     dim(FETCH_SIZE)
     D   Cust                         4P 0
     D   Name                        25A
     D   City                        15A

     D FetchLen        ds                  qualified
     D                                     dim(FETCH_SIZE)
     D   Cust                              like(SQLINTEGER)
     D   Name                              like(SQLINTEGER)
     D   City                              like(SQLINTEGER)

     D x               s             10I 0

      /free
           CEE4RAGE(%paddr(Cleanup): *OMIT);

         // --------------------------------------------------------
         // Allocate new SQL Environment & Connect to local database
         //   with the following attributes:
         //     Commit = *NONE
         //     Naming = *SYS
         // --------------------------------------------------------

           SQLAllocEnv(env);
           SQLSetEnvAttrI( env
                         : SQL_ATTR_OUTPUT_NTS
                         : SQL_FALSE
                         : 0 );

           SQLAllocConnect(env: conn);

           rc = SQLSetConnectAttrI( conn
                                  : SQL_ATTR_DBC_SYS_NAMING
                                  : SQL_TRUE
                                  : 0 );
           check_error(rc: env: conn: SQL_NULL_HSTMT
                      : 'SQLSetConnectAttr(SQL_ATTR_DBC_SYS_NAMING)');

           rc = SQLConnect( conn
                          : '*LOCAL'
                          : SQL_NTS
                          : *NULL
                          : SQL_NTS
                          : *NULL
                          : SQL_NTS );
           check_error(rc: env: conn: SQL_NULL_HSTMT: 'SQLConnect');


         // --------------------------------------------------------
         //  Create a statement for fetching 10 records at once
         //   (SQL_ATTR_FOR_FETCH_ONLY isn't required, but will
         //     improve performance.)
         // --------------------------------------------------------

          rc = SQLAllocStmt( conn : stmt );
          check_error(rc: env: conn: SQL_NULL_HSTMT: 'SQLAllocStmt');

          rc = SQLSetStmtAttrI( stmt
                              : SQL_ATTR_FOR_FETCH_ONLY
                              : SQL_TRUE
                              : 0 );
          check_error(rc: env: conn: stmt
                     : 'SQLSetStmtAttr(SQL_ATTR_FOR_FETCH_ONLY)');

          rc = SQLSetStmtAttrI( stmt
                              : SQL_ATTR_ROWSET_SIZE
                              : FETCH_SIZE
                              : 0 );
          check_error(rc: env: conn: stmt
                     : 'SQLSetStmtAttr(SQL_ATTR_ROWSET_SIZE)');

         // --------------------------------------------------------
         //  Run SQL Query in statement
         // --------------------------------------------------------

          rc = SQLExecDirect( stmt
                            : 'select CustNo,Name,City'
                            + ' from CUSTFILE'
                            + ' order by City'
                            : SQL_NTS );
          check_error(rc: env: conn: stmt: 'SQLExecDirect');

         // --------------------------------------------------------
         //  Tell SQL which variables will receive the data from the
         //  database columns
         //
         //  a) Variables bound to must be arrays.
         //  b) Arrays must contain all 3 columnss, contiguous,
         //        like they're specified on the SELECT statement.
         //  c) the resulting lengths must also be in an array,
         //        laid out the same way as the columns.
         // --------------------------------------------------------
          rc = SQLBindCol( stmt
                         : 1
                         : SQL_DECIMAL
                         : %addr(Row(1).Cust)
                         : %len(Row(1).Cust)*256 + %decpos(Row(1).Cust)
                         : fetchlen(1).Cust );
          check_error( rc: env: conn: stmt: 'SQLBindCol(1)');

          rc = SQLBindCol( stmt
                         : 2
                         : SQL_CHAR
                         : %addr(Row(1).Name)
                         : %size(Row.Name)
                         : fetchlen(1).Name );
          check_error(rc: env: conn: stmt: 'SQLBindCol(2)');

          rc = SQLBindCol( stmt
                         : 3
                         : SQL_CHAR
                         : %addr(Row(1).City)
                         : %size(Row.City)
                         : fetchlen(1).City );
          check_error(rc: env: conn: stmt: 'SQLBindCol(3)');

         // --------------------------------------------------------
         //   Fetch results of query & print them
         // --------------------------------------------------------

          dow '1';

              rc = SQLExtendedFetch( stmt
                                   : SQL_FETCH_NEXT
                                   : 0
                                   : count
                                   : status );
              check_error(rc: env: conn: stmt: 'SQLExtendedFetch()');

              if (rc = SQL_NO_DATA_FOUND);
                 leave;
              endif;

              for x = 1 to count;
                  PrintMe.Cust = %char(Row(x).Cust);
                  PrintMe.Name = Row(x).Name;
                  PrintMe.City = Row(x).City;
                  write QSYSPRT PrintMe;
              endfor;

          enddo;

         // --------------------------------------------------------
         //   Clean up & End program
         // --------------------------------------------------------
         Cleanup();
         *inlr = *on;

      /end-free


      *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      * check_error(): check for an error in the last SQL statement
      *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     P check_error     B
     D check_error     pi
     D    rc                         10I 0 value
     D    env                              like(SQLHENV)  value
     D    conn                             like(SQLHDBC)  value
     D    stmt                             like(SQLHSTMT) value
     D    apiname                    50A   varying const

     D QMHSNDPM        PR                  ExtPgm('QMHSNDPM')
     D   MessageID                    7A   Const
     D   QualMsgF                    20A   Const
     D   MsgData                    512A   Const
     D   MsgDtaLen                   10I 0 Const
     D   MsgType                     10A   Const
     D   CallStkEnt                  10A   Const
     D   CallStkCnt                  10I 0 Const
     D   MessageKey                   4A
     D   ErrorCode                32702A   options(*varsize)

     D MsgKey          s              4A
     D ErrCode         ds
     D   BytesProv                   10I 0 inz(0)
     D   BytesAvail                  10I 0

     D sqlstate        s              6A
     D sqlcode         s                   like(SQLINTEGER)
     D msgtype         s             10A   inz('*DIAG')
     D errmsg          s            513A
     D msgbytes        s                   like(SQLSMALLINT)

      /free
          if ( rc=SQL_NO_DATA_FOUND or rc=SQL_SUCCESS );
              return;
          endif;

          SQLError( env
                  : conn
                  : stmt
                  : sqlstate
                  : sqlcode
                  : errmsg
                  : %size(Errmsg)
                  : msgbytes );

          errMsg = apiname +' failed. RC=' + %char(RC)
                 + ', SQLCODE=' + %char(sqlcode)
                 + ', SQLSTATE=' + %str(%addr(sqlstate));

          if (rc < SQL_SUCCESS);
              Cleanup();
              msgtype = '*ESCAPE';
          endif;

          QMHSNDPM( 'CPF9897'
                  : 'QCPFMSG   *LIBL'
                  : %str(%addr(ErrMsg))
                  : %len(%str(%addr(ErrMsg)))
                  : MsgType
                  : '*PGMBDY'
                  : 1
                  : MsgKey
                  : ErrCode );

          return;
      /end-free
     P                 E


      *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      * Cleanup(): Deallocate/Disconnect statement handles
      *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     P cleanup         B
     D cleanup         pi
      /free
          if (stmt <> SQL_NULL_HSTMT);
             SQLFreeStmt( stmt : SQL_DROP );
             stmt = SQL_NULL_HSTMT;
          endif;

          if (conn <> SQL_NULL_HDBC);
             SQLDisconnect( conn );
             SQLFreeConnect( conn );
             conn = SQL_NULL_HDBC;
          endif;

          if (env <> SQL_NULL_HENV);
             SQLFreeEnv( env );
             env = SQL_NULL_HENV;
          endif;
      /end-free
     P                 E
